"""
Selectors for querying and modifying playlists
"""
from functools import partial
from typing import Iterable

from spotify_model import Paging, PlaylistTrack, SimplifiedPlaylist, SimplifiedTrack
from spotipy import Spotify

from .user import current_user
from .util import chunk, exhaust


def playlist_current_user_create(client: Spotify, user_id: str, name: str) -> SimplifiedPlaylist:
    """
    Create a playlist for a user

    https://developer.spotify.com/documentation/web-api/reference/#endpoint-create-playlist
    """
    return SimplifiedPlaylist(**client.user_playlist_create(user_id, name))


def playlist_current_user_all(client: Spotify) -> Iterable[SimplifiedPlaylist]:
    """
    Get all playlists that belong to the current user
    """

    def _playlists(limit: int, offset: int) -> Paging:
        return Paging(**client.current_user_playlists(limit, offset))

    for playlist in exhaust(_playlists):
        yield SimplifiedPlaylist(**playlist)


def playlist_current_user_find(playlists: Iterable[SimplifiedPlaylist], name: str) -> Iterable[str]:
    """
    Find all playlists for the current user that match a specific name
    """
    for playlist in playlists:
        if playlist.name == name:
            yield playlist


def playlist_current_user_assure(
    client: Spotify, playlists: Iterable[SimplifiedPlaylist], name: str
) -> Iterable[SimplifiedPlaylist]:
    """
    Find (or create if it doesn't exist) all playlists for the current user that match a specific name
    """
    found = False

    for playlist in playlist_current_user_find(playlists, name):
        found = True
        yield playlist

    if not found:
        current_user_id = current_user(client).spotify_id
        yield playlist_current_user_create(client, current_user_id, name)


def playlist_current_user_replace(
    client: Spotify, playlist_id: str, tracks: Iterable[SimplifiedTrack], chunk_size: int = 100
) -> None:
    """
    Replace all tracks in a playlist with the new provided tracks
    """
    all_tracks = [t.spotify_id for t in tracks]

    first = True

    for track_chunk in chunk(all_tracks, chunk_size):
        if first:
            client.playlist_replace_items(playlist_id, track_chunk)
            first = False
            continue

        client.playlist_add_items(playlist_id, track_chunk)


def playlist_tracks(client: Spotify, playlist_ids: Iterable[str]) -> Iterable[PlaylistTrack]:
    """
    Given a playlist_id, fetch all songs currently in the playlist
    """

    def _playlist_tracks(playlist_id: str, limit: int, offset: int) -> Paging:
        return Paging(**client.playlist_items(playlist_id, limit=limit, offset=offset))

    for playlist_id in playlist_ids:
        playlist_function = partial(_playlist_tracks, playlist_id=playlist_id)

        for track in exhaust(playlist_function):
            yield PlaylistTrack(**track).track
